module CONTROLLER (CLK, NEW_KEY, ENABLE, X_COORD, Y_COORD, SCAN_CODE, RGB_VAL) ;
input CLK;
input NEW_KEY, ENABLE;
input [9:0] X_COORD, Y_COORD;
input [7:0] SCAN_CODE;
output [2:0] RGB_VAL;
reg [2:0] RGB_VAL;
reg [9:0] X_VAL_LOW, X_VAL_HIGH, Y_VAL_LOW, Y_VAL_HIGH, X_VAL_LINE, Y_VAL_LINE, X_LINE1, Y_LINE1;
reg [7:0] COUNTER;
reg [9:0] COUNTER2;
reg NEW_KEY_HOLD;

always @(posedge CLK) begin
	// reset step
	if(SCAN_CODE == 'h76) begin
		X_VAL_LOW <= 300;
		X_VAL_HIGH <= 350;
		Y_VAL_LOW <= 250;
		Y_VAL_HIGH <= 300;
		X_VAL_LINE <= 310;
		Y_VAL_LINE <= 0;
		X_LINE1 <= 310;
		Y_LINE1 <= 240;
		COUNTER <= 0;
		COUNTER2 <= 0;
		NEW_KEY_HOLD <= 0;
	end
	// prepare to check keys
	if(ENABLE) begin		// if the logical and of HRGB and VRGB is 1, do something
		// increment counters
		COUNTER <= COUNTER + 1;		
		if(COUNTER == 0) begin
			COUNTER2 <= COUNTER2 + 1;
		end
		
		// check to see if we have a new key
		if(NEW_KEY_HOLD == 0) begin
			NEW_KEY_HOLD <= NEW_KEY;
		end
		
		// if we have a new key, go to work, process input
		else begin
			if(COUNTER == 0)begin
				NEW_KEY_HOLD <= 0;
				case (SCAN_CODE)
				'h75: begin		// up arrow
					if(Y_VAL_LOW == 160) begin
						Y_VAL_LOW <= 160;
					end
					else begin
						Y_VAL_LOW <= Y_VAL_LOW - 1;
					end
					if(Y_VAL_HIGH == 210) begin
						Y_VAL_HIGH <= 210;
					end
					else begin
						Y_VAL_HIGH <= Y_VAL_HIGH - 1;
					end 
				end
				'h72: begin		// down arrow
					if(Y_VAL_LOW == 430) begin
						Y_VAL_LOW <= 430;
					end
					else begin
						Y_VAL_LOW <= Y_VAL_LOW + 1;
					end
					if(Y_VAL_HIGH == 480) begin
						Y_VAL_HIGH <= 480;
					end
					else begin
						Y_VAL_HIGH <= Y_VAL_HIGH + 1;
					end 
				end
				'h6b: begin		// left arrow
					if(X_VAL_LOW == 150) begin
						X_VAL_LOW <= 150;
					end
					else begin
						X_VAL_LOW <= X_VAL_LOW - 1;
					end
					if(X_VAL_HIGH == 200) begin
						X_VAL_HIGH <= 200;
					end
					else begin
						X_VAL_HIGH <= X_VAL_HIGH - 1;
					end 
				end
				'h74: begin		// right arrow
					if(X_VAL_LOW == 440) begin
						X_VAL_LOW <= 440;
					end
					else begin
						X_VAL_LOW <= X_VAL_LOW + 1;
					end
					if(X_VAL_HIGH == 490) begin
						X_VAL_HIGH <= 490;
					end
					else begin
						X_VAL_HIGH <= X_VAL_HIGH + 1;
					end 
				end
				'h76: begin		// escape key - return to default settings
					X_VAL_LOW <= 300;
					X_VAL_HIGH <= 350;
					Y_VAL_LOW <= 250;
					Y_VAL_HIGH <= 300;
					X_VAL_LINE <= 310;
					Y_VAL_LINE <= 0;
					X_LINE1 <= 310;
					Y_LINE1 <= 240;
				end
				'h29: begin		// space bar
				end
				default: begin		// any other key
				end
				endcase
			end // end if counter == 0
		end // end else new key
		
		// general layout
		if(X_COORD < 150 || X_COORD > 490) begin	// grass
			RGB_VAL <= 'b010;
		end 
		
		// roadway
		else begin
		
		// draw the vehicle		
		if(X_COORD < X_VAL_HIGH && X_COORD > X_VAL_LOW && Y_COORD > Y_VAL_LOW && Y_COORD < Y_VAL_HIGH) begin
			RGB_VAL <= 'b100;
		end
		else begin
		// draw and update position for the centerline
		if(X_COORD > 310 && X_COORD < 330) begin
			if(COUNTER2 == 0) begin
				if(Y_VAL_LINE > 479) begin
					Y_VAL_LINE <= 0;
				end
				else begin
					Y_VAL_LINE <= Y_VAL_LINE + 1;
				end
				if(Y_LINE1 > 479) begin
					Y_LINE1 <= 0;
				end
				else begin
					Y_LINE1 <= Y_LINE1 + 1;
				end
			end // end counter2 == 0
			
			// check first middle stripe
			if((Y_COORD > Y_VAL_LINE) && (Y_COORD < (Y_VAL_LINE + 50))) begin
				if(X_COORD < X_VAL_HIGH && X_COORD > X_VAL_LOW && Y_COORD > Y_VAL_LOW && Y_COORD < Y_VAL_HIGH) begin
				end
				else begin
					RGB_VAL <= 'b111;
				end
			end // end y_COORD....
			
			// check second middle stripe
			else begin
			if((Y_COORD > Y_LINE1) && (Y_COORD < (Y_LINE1 + 50))) begin
				if(X_COORD < X_VAL_HIGH && X_COORD > X_VAL_LOW && Y_COORD > Y_VAL_LOW && Y_COORD < Y_VAL_HIGH) begin
				end
				else begin
					RGB_VAL <= 'b111;
				end
			end // end y_COORD....
			
			else begin
				RGB_VAL <= 'b0;
			end
			end // else not first stripe
		end // end X_Coord ...
		else begin
			RGB_VAL <= 'b0;
		end // end else not median
		end // end else not vehicle
		end // end roadway else

	end // enable
	//else begin			// otherwise, do nothing
	//	RGB_VAL <= 0;
	//end	
end // end always block

endmodule 
